"use strict";
/*
 * This file is part of textlint-plugin-latex2e
 *
 * textlint-plugin-latex2e is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * textlint-plugin-latex2e is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with textlint-plugin-latex2e.  If not, see <http://www.gnu.org/licenses/>.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.completeComments = exports.insertComment = exports.isParentNode = exports.isIncludedByNode = exports.isAppearedBeforeNode = exports.convertCommentToTxtNode = void 0;
const ast_node_types_1 = require("@textlint/ast-node-types");
exports.convertCommentToTxtNode = (rawText, comments) => {
    if (!comments) {
        return [];
    }
    return comments === null || comments === void 0 ? void 0 : comments.map((comment) => {
        return {
            loc: {
                start: {
                    line: comment.location.start.line,
                    column: comment.location.start.column - 1,
                },
                end: {
                    line: comment.location.end.line,
                    column: comment.location.end.column - 1,
                },
            },
            range: [comment.location.start.offset, comment.location.end.offset],
            raw: rawText.slice(comment.location.start.offset, comment.location.end.offset),
            value: comment.content,
            type: ast_node_types_1.ASTNodeTypes.Comment,
        };
    });
};
exports.isAppearedBeforeNode = (nodeRange, commentRange) => {
    return nodeRange[0] >= commentRange[1];
};
exports.isIncludedByNode = (nodeRange, commentRange) => {
    return nodeRange[0] <= commentRange[0] && nodeRange[1] >= commentRange[1];
};
exports.isParentNode = (node) => {
    const children = node.children;
    return (typeof node === "object" &&
        children !== undefined &&
        Array.isArray(children));
};
exports.insertComment = (comment, nodes) => {
    if (!nodes) {
        return exports.insertComment(comment, []);
    }
    for (let i = 0; i < nodes.length; i++) {
        // If the comment is appeared before the node, insert it before the node.
        if (exports.isAppearedBeforeNode(nodes[i].range, comment.range)) {
            nodes.splice(i, 0, comment);
            return nodes;
        }
        // If the comment is included in the node, try to insert it recursively.
        if (exports.isIncludedByNode(nodes[i].range, comment.range)) {
            if (exports.isParentNode(nodes[i])) {
                nodes[i].children = exports.insertComment(comment, nodes[i].children);
                return nodes;
            }
            // `Parbreak` has no children, even though the range of
            // `Parbreak` includes the range of comment
            // if comment is surrounding by the line break.
            // But the parbreak becomes null, so this condition does not used.
            throw Error("Unexpected node is given. Is the syntax correct?");
        }
    }
    // If the comment is not inserted, it would be appeared after all nodes.
    nodes.push(comment);
    return nodes;
};
// Mapping all comments to the given AST.
exports.completeComments = (comments) => (rawText) => (root) => {
    if (comments.length === 0) {
        return root;
    }
    const textlintComments = exports.convertCommentToTxtNode(rawText, comments);
    const copiedRoot = JSON.parse(JSON.stringify(root));
    for (const comment of textlintComments) {
        copiedRoot.children = exports.insertComment(comment, copiedRoot.children);
    }
    return copiedRoot;
};
